from dataclasses import asdict
import json
from typing import Type, TypeVar

from ai_emotion.simple_emotion import PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion
from ai_emotion.gpt import get_gpt_interface


A = TypeVar("A", PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion)
B = TypeVar("B", PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion)


def convert_emotion(
    emotion_a: A,
    type_b: Type[B],
    openai_api_key: str,
    context: str | None = None,
) -> B:
    interface = get_gpt_interface(openai_api_key)
    prompt = ( 
f"""
{emotion_a.description()}

{type_b.description()}

Convert the following {emotion_a.__class__.__name__} to a {type_b.__name__}:
{asdict(emotion_a)}
"""
    )
    if context:
        prompt += f"The context for this emotion is: {context}"
    response = interface.say(prompt)
    return type_b(**json.loads(response))



from dataclasses import asdict

from ai_emotion.gpt import get_gpt_interface
from ai_emotion.simple_emotion import VectoralEmotion, PlutchikEmotion, PhysiologicalEmotion


def describe_emotion(
    emotion: PhysiologicalEmotion | PlutchikEmotion | VectoralEmotion,
    openai_api_key: str,
    context: str | None = None,
) -> str:
    interface = get_gpt_interface(openai_api_key)
    prompt = (
f"""
{emotion.__class__.description()}

Describe the following emotion in words (e.g. "Nervous about an upcoming presentation, and worried that I may perform poorly.").
Emotion:
{asdict(emotion)}
"""
    )
    if context:
        prompt += (
f"""
The context is:
{context}
"""
        )
    response = interface.say(prompt)
    return response



import json
from typing import Type, TypeVar

from ai_emotion.gpt import get_gpt_interface
from ai_emotion.simple_emotion import VectoralEmotion, PlutchikEmotion, PhysiologicalEmotion


A = TypeVar("A", PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion)


def generate_emotion(
    emotion_type: Type[A],
    prompt: str,
    openai_api_key: str,
) -> A:
    interface = get_gpt_interface(openai_api_key)
    response = interface.say( 
f"""
{emotion_type.description()}

Create a {A.__name__} that corresponds to the following description:
{prompt}
"""
    )
    return emotion_type(**json.loads(response))



from dataclasses import dataclass, fields
import numpy as np
import pandas as pd
from typing import Self


@dataclass
class EmotionVector:
    def to_series(self):
        return pd.Series(self.__dict__)

    @classmethod
    def from_series(cls, series: pd.Series) -> Self:
        # convert a series back to an instance of the class
        kwargs = {field.name: series[field.name] for field in fields(cls)}
        return cls(**kwargs)

    @classmethod
    def description(cls) -> str:
        return "Description of fields in emotion vector."

    def length(self):
        series = self.to_series()
        return np.linalg.norm(series)

    def normalize(self):
        series = self.to_series()
        normalized_series = series / self.length()
        return self.from_series(normalized_series)

    def subtract(self, other: Self) -> Self:
        if not isinstance(other, type(self)):
            raise ValueError("Subtraction requires another instance of the same class")
        series1 = self.to_series()
        series2 = other.to_series()
        return self.from_series(series1 - series2)

    def add(self, other: Self) -> Self:
        if not isinstance(other, type(self)):
            raise ValueError("Addition requires another instance of the same class")
        series1 = self.to_series()
        series2 = other.to_series()
        return self.from_series(series1 + series2)



from dataclasses import dataclass

from ai_emotion.simple_emotion import VectoralEmotion, PlutchikEmotion, PhysiologicalEmotion


SimpleEmotion = PlutchikEmotion | PhysiologicalEmotion | VectoralEmotion


# TODO
@dataclass
class ComplexEmotion:
    emotions: list[tuple[float, SimpleEmotion]]



from gpt_interface import GptInterface


def get_gpt_interface(openai_api_key: str) -> GptInterface:
    interface = GptInterface(
        openai_api_key=openai_api_key,
        model='gpt-4',
        json_mode=True,
    )
    interface.set_system_message(
        """
        Create a possible conversion between emotion representations that seems reasonable.
        Give your best guesses, and respond with the emotion in JSON format.
        Don't add any extra information in your response.
        """
    )
    return interface



from dataclasses import dataclass
from typing import cast, Self
from random import random

from ai_emotion.emotion_vector import EmotionVector


@dataclass
class PhysiologicalEmotion(EmotionVector):
    heart_rate: float
    breathing_rate: float
    hair_raised: float
    blood_pressure: float
    body_temperature: float
    muscle_tension: float
    pupil_dilation: float
    gi_blood_flow: float
    amygdala_blood_flow: float
    prefrontal_cortex_blood_flow: float
    muscle_blood_flow: float
    genitalia_blood_flow: float
    smile_muscle_activity: float
    brow_furrow_activity: float
    lip_tightening: float
    throat_tightness: float
    mouth_dryness: float
    voice_pitch_raising: float
    speech_rate: float
    cortisol_level: float
    adrenaline_level: float
    oxytocin_level: float

    @classmethod
    def description(cls) -> str:
        return (
"""
PhysiologicalEmotion is a vector with the following values, where each value is a float from 0 to 1 inclusive, representing the intensity of the physiological response:
- heart_rate
- breathing_rate
- hair_raised
- blood_pressure
- body_temperature
- muscle_tension
- pupil_dilation
- gi_blood_flow
- amygdala_blood_flow
- prefrontal_cortex_blood_flow
- muscle_blood_flow
- genitalia_blood_flow
- smile_muscle_activity
- brow_furrow_activity
- lip_tightening
- throat_tightness
- mouth_dryness
- voice_pitch_raising
- speech_rate
- cortisol_level
- adrenaline_level
- oxytocin_level
"""
        )

    @classmethod
    def random(cls) -> Self:
        return PhysiologicalEmotion(
            heart_rate=random(),
            breathing_rate=random(),
            hair_raised=random(),
            blood_pressure=random(),
            body_temperature=random(),
            muscle_tension=random(),
            pupil_dilation=random(),
            gi_blood_flow=random(),
            amygdala_blood_flow=random(),
            prefrontal_cortex_blood_flow=random(),
            muscle_blood_flow=random(),
            genitalia_blood_flow=random(),
            smile_muscle_activity=random(),
            brow_furrow_activity=random(),
            lip_tightening=random(),
            throat_tightness=random(),
            mouth_dryness=random(),
            voice_pitch_raising=random(),
            speech_rate=random(),
            cortisol_level=random(),
            adrenaline_level=random(),
            oxytocin_level=random(),
        )


@dataclass
class PlutchikEmotion(EmotionVector):
    joy: float
    sadness: float
    trust: float
    disgust: float
    fear: float
    anger: float
    surprise: float
    anticipation: float

    @classmethod
    def description(cls) -> str:
        return (
"""
PlutchikEmotion is a vector with the following values, where each value is a float from 0 to 1 inclusive, representing the intensity of the emotion:
- joy
- sadness
- trust
- disgust
- fear
- anger
- surprise
- anticipation
"""
        )

    @classmethod
    def random(cls) -> Self:
        return PlutchikEmotion(
            joy=random(),
            sadness=random(),
            trust=random(),
            disgust=random(),
            fear=random(),
            anger=random(),
            surprise=random(),
            anticipation=random(),
        )


@dataclass
class VectoralEmotion(EmotionVector):
    valence: float
    arousal: float
    control: float

    @classmethod
    def description(cls) -> str:
        return (
"""
VectoralEmotion is a vector with the following values, where each value is a float from -1 to 1 inclusive, representing the intensity of the emotion:
- valence (negative is bad, positive is good)
- arousal (negative is calm, positive is excited)
- control (negative is submissive, positive is dominant)
"""
        )

    @classmethod
    def random(cls) -> Self:
        return VectoralEmotion(
            valence=random()*2-1,
            arousal=random()*2-1,
            control=random()*2-1,
        )



from dataclasses import asdict
import json
from typing import TypeVar

from ai_emotion.simple_emotion import PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion
from ai_emotion.gpt import get_gpt_interface


A = TypeVar("A", PhysiologicalEmotion, PlutchikEmotion, VectoralEmotion)


def transition_emotion(
    emotion: A,
    context: str,
    prompt: str,
    openai_api_key: str,
) -> A:
    interface = get_gpt_interface(openai_api_key)
    response = interface.say(
f"""
{emotion.description()}

As a cause of this:
{context}

A person feels this emotion:
{asdict(emotion)}

Determine this person's updated emotion after the following occurs:
{prompt}

Reply as a new {type(emotion).__name__}.
Make sure JSON format uses double quotes for keys.
"""
    )
    return emotion.__class__(**json.loads(response))
